// Ejemplo FreeImage 5
// Leer pixels
// Abril 2023, Salvador Pozo, Con Clase

#include <iostream>
#include <iomanip>
#include <Freeimage.h>
#include <cstring>

int main(int argc, char**argv)
{
    FIBITMAP* dib;
	char *lpszPathName;
    FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
    BYTE *bits;
    unsigned bpp;
    unsigned rojo, verde, azul;
    unsigned nBytes, nPixels;
    RGBQUAD *pal;

	if(argc < 2) {
        printf("Usar: FreeImage005 <fichero>\n");
        return 0;
	}
	lpszPathName = argv[1];

	fif = FreeImage_GetFileType(lpszPathName, 0);
	if(fif == FIF_UNKNOWN) {
		fif = FreeImage_GetFIFFromFilename(lpszPathName);
	}
	if((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
		dib = FreeImage_Load(fif, lpszPathName, 0);
		std::cout << "Fichero: " << lpszPathName << std::endl;

		if(FreeImage_HasPixels(dib)) {
            std::cout << "Tiene pixels" << std::endl;
            bpp = FreeImage_GetBPP(dib);
            pal = FreeImage_GetPalette(dib);
            if(!pal) {
                std::cout << "Mapa de bits sin paleta" << std::endl;
                bits = FreeImage_GetBits(dib);
                rojo = FreeImage_GetRedMask(dib);
                verde = FreeImage_GetGreenMask(dib);
                azul = FreeImage_GetBlueMask(dib);
                std::cout << "BPP: " << bpp << std::endl;
                std::cout << "Mascara Rojo:  " << std::hex << rojo << std::endl;
                std::cout << "Mascara verde: " << std::hex << verde << std::endl;
                std::cout << "Mascara Azul:  " << std::hex << azul << std::endl;
                std::cout << "Anchura: " << FreeImage_GetWidth(dib) << std::endl;
                std::cout << "Altura: " << FreeImage_GetHeight(dib) << std::endl;
                nBytes = bpp/8;
                std::cout << "Primeros diez pixels de primera fila: " << std::endl;
                for(unsigned int y = 0; y < 10; y++) {
                    for(unsigned int x = 0; x < 10; x++) {
                        DWORD valor=0;
                        for(unsigned int i=0; i<nBytes; i++) {
                            valor <<= 8;
                            valor |= bits[y*(nBytes*FreeImage_GetWidth(dib))+x*nBytes+i];
                        }
                        std::cout << std::hex << (int)valor << ",";
                        // Usando FreeImage_GetPixelColor:
                        /*RGBQUAD valor;
                        FreeImage_GetPixelColor(dib, x, y, &valor);
                        std::cout << std::hex << (int)valor.rgbRed << (int)valor.rgbGreen << (int)valor.rgbBlue << ", ";*/
                    }
                    std::cout << std::endl;
                }
            } else {
                std::cout << "Mapa de bits con paleta" << std::endl;
                bits = FreeImage_GetBits(dib);
                std::cout << "BPP: " << bpp << std::endl;
                std::cout << "Anchura: " << FreeImage_GetWidth(dib) << std::endl;
                std::cout << "Altura: " << FreeImage_GetHeight(dib) << std::endl;
                nPixels = 8/bpp;
                BYTE mask=0;
                for(unsigned int i=0; i<bpp; i++) mask |= 0x80>>i;
                std::cout << "Mascara: " << std::hex << (int)mask << std::endl;
                std::cout << "Primeros diez pixels de primera fila: " << std::endl;
                for(unsigned int y = 0; y < 10; y++) {
                    for(unsigned int x = 0; x < 10/nPixels; x++) {
                        BYTE b = bits[y*(FreeImage_GetWidth(dib)/nPixels)+x];
                        for(unsigned int i=0; i<nPixels; i++) {
                            int index = (int)((b & mask)>>(8-bpp));
                            //std::cout << std::hex << index << ":";
                            std::cout << std::hex << (int)pal[index].rgbRed << "|" << (int)pal[index].rgbGreen << "|" << (int)pal[index].rgbBlue << ", ";
                            b <<= bpp;
                        }
                        // Usando FreeImage_GetPixelIndex
                       /* BYTE b;
                        FreeImage_GetPixelIndex(dib, x, y, &b);
                        RGBQUAD valor = pal[b];
                        std::cout << std::hex << (int)valor.rgbRed << (int)valor.rgbGreen << (int)valor.rgbBlue << ", ";*/
                    }
                    std::cout << std::endl;
                }
            }
		}
    }
	FreeImage_Unload(dib);

    return 0;
}

